#!/usr/bin/env python
# Winston Fernandes
#
""" This file is responsible to parse the input repository files and generate catalog.xml file from it """

from __future__ import print_function

import HTMLParser
import subprocess
import urllib2
import os, re
import zipfile
import xml.etree.ElementTree as ET
import sys


def get_files(folder_path):
    ret = []
    for path, subdirs, files in os.walk(folder_path):
        for name in files:
            ret.append(os.path.join(path, name))
    return ret


def extract_files(files, dir_name):

    """ extract the input zip files to given directory """
    
    os.chdir(dir_name)
    extracted_files = []
    for file in files:
        if zipfile.is_zipfile(file):
            zip_ref = zipfile.ZipFile(file)
            temp_file = file[:-4]
            (head1, tail1) = os.path.split(file)
            (head2, tail2) = os.path.split(temp_file)
            zip_map[tail2] = tail1
            temp_file = temp_file + "/"
            zip_ref.extractall(temp_file)
            extracted_files.append(temp_file)
            zip_ref.close()
    return extracted_files


def get_metadata_files(files):

    """ Mainly to get the names of metadata files.
        1.  Parse the index.xml (assuming this will exist in all cases) and get the vendor-index.xml file name
        2.  Parse the vendor-index.xml file to get the metadata file name """

    extracted_metadata_files = set()
    vendor_xml_name = None
    for file in files:
        if os.path.exists(file + '/index.xml'):
            index_file = file + 'index.xml'
            tree = ET.parse(index_file)
            for elem in tree.getroot():
                for subelem in elem:
                    if subelem.tag == 'indexfile':
                        vendor_xml_name = subelem.text
            vendor_file = file + vendor_xml_name
            tree = ET.parse(vendor_file)
            for metadata in tree.findall('metadata'):
                if metadata.find('url') is not None:
                    meta_name = metadata.find('url').text
                    meta_file_name = file + meta_name
                else:
                    meta_file_name = file + "metadata.zip"
                extracted_metadata_files.add(meta_file_name)
    return extracted_metadata_files


def indent(elem, level=0):
    i = "\n" + level * "  "
    j = "\n" + (level - 1) * "  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for subelem in elem:
            indent(subelem, level + 1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = j
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = j
    return elem


def get_zip_information(file):
    """ Parse the input vmware.xml (assuming the input must be vmware.xml)
        and adding all the required elements to a dictionary (package)."""

    # TODO : Need to add changes to optimize the scenario when two vmware.xml files
    #  for same component with different platforms

    tree = ET.parse(file)
    root = tree.getroot()
    vibList = []
    package_type = None
    path, file_name = os.path.split(file)
    if file_name == 'vmware.xml':
        (file_path, tail) = os.path.split(path)
    else:
        file_path = file.replace("/metadata/vmware.xml", "")
    (head, tail) = os.path.split(file_path)
    for bulletin in tree.findall('bulletin'):
        platforms = bulletin.find('platforms')
        if bulletin.find('componentNameSpec') is not None:
            package_type = bulletin.find('componentNameSpec')
        for viblist in bulletin.findall('vibList'):
            for vib in viblist.findall('vib'):
                tempVib = {}
                name = vib.find('name')
                tempVib['name'] = name
                version = vib.find('version')
                tempVib['version'] = version
                reboot = vib.find('postInstall/rebootRequired')
                tempVib['reboot'] = reboot
                vendor = vib.find('vendor')
                tempVib['vendor'] = vendor
                tempVib['platform'] = platforms
                tempVib['type'] = package_type
                vibList.append(tempVib)
    package[tail] = vibList


package = {}
zip_map = {}
path = sys.argv[1]
print("path", path)
zip_files = get_files(path)
files = extract_files(zip_files, path)
metadata_zips = get_metadata_files(files)
metadata_files = extract_files(metadata_zips, path)
for meta_file in metadata_files:
    get_zip_information(meta_file + "vmware.xml")
catalog = ET.Element('catalog')
for package, vibs in package.items():
    packageElement = ET.SubElement(catalog, 'component')
    packageName = ET.SubElement(packageElement, 'name')
    packageName.text = package
    packagePath = ET.SubElement(packageElement, 'path')
    packagePath.text = zip_map[package]
    vibListElement = ET.SubElement(packageElement, 'vibList')
    for vib in vibs:
        print(vib)
        vibElement = ET.SubElement(vibListElement, 'vib')
        packageType = ET.Element("packageType")
        vibElement.insert(0, vib['name'])
        vibElement.insert(1, vib['version'])
        vibElement.insert(2, vib['vendor'])
        vibElement.insert(3, vib['reboot'])
        if vib['type'] is not None:
            packageType.text = "component"
        else:
            packageType.text = "bulletin"
        vibElement.insert(4, packageType)
        vibElement.insert(5, vib['platform'])

indent(catalog)
doc = ET.ElementTree(catalog)
doc.write(open('catalog.xml', 'wb'))
